# Generated by roxygen2: do not edit by hand

export(GeomAlluvium)
export(GeomFlow)
export(GeomLode)
export(GeomStratum)
export(StatAlluvium)
export(StatFlow)
export(StatStratum)
export(data_to_alluvium)
export(geom_alluvium)
export(geom_flow)
export(geom_lode)
export(geom_stratum)
export(is_alluvia_form)
export(is_alluvial)
export(is_alluvial_alluvia)
export(is_alluvial_lodes)
export(is_lodes_form)
export(lode_backfront)
export(lode_backward)
export(lode_forward)
export(lode_frontback)
export(lode_leftright)
export(lode_leftward)
export(lode_rightleft)
export(lode_rightward)
export(lode_zagzig)
export(lode_zigzag)
export(positions_to_flow)
export(self_adjoin)
export(stat_alluvium)
export(stat_flow)
export(stat_stratum)
export(to_alluvia)
export(to_alluvia_form)
export(to_lodes)
export(to_lodes_form)
import(ggplot2)
importFrom(rlang,"%||%")
importFrom(rlang,.data)
importFrom(rlang,enexpr)
importFrom(rlang,enexprs)
importFrom(rlang,enquo)
importFrom(rlang,enquos)
importFrom(rlang,have_name)
importFrom(rlang,is_character)
importFrom(rlang,is_empty)
importFrom(rlang,is_integerish)
importFrom(rlang,is_quosures)
importFrom(rlang,quo_name)
importFrom(rlang,quos)
importFrom(tidyselect,eval_select)
importFrom(tidyselect,vars_pull)
importFrom(tidyselect,vars_select)
